#
# Authors:  
#          Nathan Hamiel
#          Seth Law
#
# Copyright (c) 2011 RAFT Team
#
# This file is part of RAFT.
#
# RAFT is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# RAFT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RAFT.  If not, see <http://www.gnu.org/licenses/>.
#
# This is the JavaScript analysis module.

import re
import cgi

#ToDo: add window.open

def find_eval(content):
    """ Search for the use of JavaScript eval statements in the supplied content """
    
    format = "<h3>JavaScript Evals</h3>\n"
    #ToDo: Should be rewritten to allow any number of spaces between the statement and the (
    pattern = re.compile("eval\(", re.I)
    for line in content.splitlines():
        if pattern.search(line):
            format += cgi.escape(line)
            
    return(format)
    
def find_comments(content):
    """ Search for JavaScript comments """
    
    format = "<h3>Javascript Comments</h3>\n"
    pattern = re.compile("^\s*//",re.I)
    start_comment = re.compile("/\*",re.I)
    end_comment = re.compile("\*/",re.I)
    in_comment = 0
    
    
    for line in content.splitlines():
        if in_comment == 1:
            format += cgi.escape(line) + "<br>"
            if end_comment.search(line):
                in_comment = 0
        elif start_comment.search(line):
            format += cgi.escape(line) + "<br>"
            if not end_comment.search(line):
                in_comment = 1
        elif pattern.search(line):
            format += cgi.escape(line) + "<br>"
        
    return(format)
    
def doc_write(content):
    """ Identify document write functions in the supplied content. document.write, document.writeln """
    
    #ToDo: Should be rewritten to allow any number of spaces between the statement and the (
    format = "<h3>JavaScript Writes</h3>\n"
    pattern = re.compile("document\.write\(|document\.writeln\(", re.I)
    
    for line in content.splitlines():
        if pattern.search(line):
            format += cgi.escape(line)
    
    return(format)
    

def doc_location(content):
    """ Identify document location setting functions in the supplied content. .location, .href """
    
    format = "<h3>JavaScript Location Setting</h3>\n"
    pattern = re.compile("\.location(\s*=|\.href\s*=)", re.I)
    
    for line in content.splitlines():
        if pattern.search(line):
            format += cgi.escape(line)
    
    return(format)
    

def innerHTML(content):
    """ Identify the setting of innerHTML in the supplied content. """
    
    format = "<h3>JavaScript innerHTML</h3>\n"
    pattern = re.compile("\.innerHTML", re.I)
    
    for line in content.splitlines():
        if pattern.search(line):
            format += cgi.escape(line)
    
    return(format)
