#
# Implements a web page for ODM fuzzing
#
# Author: Gregory Fleischer (gfleischer@gmail.com)
#
# Copyright (c) 2011 RAFT Team
#
# This file is part of RAFT.
#
# RAFT is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# RAFT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RAFT.  If not, see <http://www.gnu.org/licenses/>.
#

import PyQt4
from PyQt4 import QtWebKit, QtNetwork
from PyQt4.QtCore import *

class DomFuzzerWebPage(QtWebKit.QWebPage):
    def __init__(self, framework, callbackLogger, parent = None):
        QtWebKit.QWebPage.__init__(self, parent)
        self.framework = framework
        self.callbackLogger = callbackLogger
        self.frameCreated.connect(self.handle_frameCreated)
        self.configure_frame(self.mainFrame())

        # TODO: make configurable?
        settings = self.settings()
        settings.setAttribute(QtWebKit.QWebSettings.PluginsEnabled, True)
        settings.setAttribute(QtWebKit.QWebSettings.JavaEnabled, True)
        settings.setAttribute(QtWebKit.QWebSettings.AutoLoadImages, False)
        settings.setAttribute(QtWebKit.QWebSettings.JavascriptCanOpenWindows, False)

    def handle_frameCreated(self, frame):
        self.configure_frame(frame)

    def configure_frame(self, frame):
        self.add_javascript_window_object(frame)
#        QObject.connect(frame, SIGNAL('loadFinished(bool)'), lambda x: self.handle_frame_loadFinished(frame, x))
        QObject.connect(frame, SIGNAL('javaScriptWindowObjectCleared()'), lambda: self.handle_javaScriptWindowObjectCleared(frame))

    def handle_frame_loadFinished(self, frame, ok):
        pass ### print('frame load finished', ok)
        pass

    def handle_javaScriptWindowObjectCleared(self, frame):
        self.add_javascript_window_object(frame)

    def add_javascript_window_object(self, frame):
        frame.addToJavaScriptWindowObject("__RAFT__", self)

    @PyQt4.QtCore.pyqtSlot(name='shouldInterruptJavaScript', result='bool')
    def shouldInterruptJavaScript(self):
        self.callbackLogger.log('*** shouldInterruptJavaScript invoked')
        return True

    def javaScriptAlert(self, frame, msg):
        self.callbackLogger.log('alert from [%s / %s]: %s' % (frame.url(), frame.requestedUrl(), msg))

    def javaScriptConfirm(self, frame, msg):
        self.callbackLogger.log('confirm from [%s / %s]: %s' % (frame.url(), frame.requestedUrl(), msg))
        # TODO: return value based on message

    def javaScriptPrompt(self, frame, msg, defaultValue, result):
        self.callbackLogger.log('prompt from [%s / %s]: %s' % (frame.url(), frame.requestedUrl(), msg))
        # TODO: return value based on message

    def javaScriptConsoleMessage(self, message, lineNumber, sourceID):
        self.callbackLogger.log('console log from [%s / %s]: %s' % (lineNumber, sourceID, message))

    def userAgentForUrl(self, url):
        return self.framework.useragent()

    def acceptNavigationRequest(self, frame, request, navigationType):
        pass ### print('navigation request (%s) -> [%s]' % (navigationType, request.url().toEncoded()))
        # TODO: should other navigation be accepted ?
        if frame and navigationType == QtWebKit.QWebPage.NavigationTypeOther:
            return True
        return True