#
# configuration dialog
#
# Authors: 
#          Gregory Fleischer (gfleischer@gmail.com)
#
# Copyright (c) 2011 RAFT Team
#
# This file is part of RAFT.
#
# RAFT is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# RAFT is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RAFT.  If not, see <http://www.gnu.org/licenses/>.
#

from PyQt4.QtCore import (Qt, SIGNAL, QObject)
from PyQt4.QtGui import *

from ui import ConfigDialog

class ConfigDialog(QDialog, ConfigDialog.Ui_configDialog):
    """ The Config Dialog """
    
    def __init__(self, framework, parent=None):
        super(ConfigDialog, self).__init__(parent)
        self.setupUi(self)

        self.framework = framework

        self.buttonBox.clicked.connect(self.handle_buttonBox_clicked)

        self.checkBoxUseProxy.stateChanged.connect(self.handle_useProxy_stateChanged)

        self.Data = None
        self.cursor = None
        self.framework.subscribe_database_events(self.db_attach, self.db_detach)

    def db_attach(self):
        self.Data = self.framework.getDB()
        self.cursor = self.Data.allocate_thread_cursor()
        self.fill_values()

    def db_detach(self):
        self.close_cursor()
        self.Data = None

    def close_cursor(self):
        if self.cursor and self.Data:
            self.cursor.close()
            self.Data.release_thread_cursor(self.cursor)
            self.cursor = None

    def fill_values(self):
        self.bhNetworkBox.setChecked(self.framework.get_raft_config_value('black_hole_network', bool))
        self.checkBoxUseProxy.setChecked(self.framework.get_raft_config_value('use_proxy', bool))
        self.confProxyEdit.setText(self.framework.get_raft_config_value('proxy_host'))
        self.confProxyPort.setText(self.framework.get_raft_config_value('proxy_port'))
        self.set_enable_proxy_edits()

    def handle_buttonBox_clicked(self, button):
        role = self.buttonBox.buttonRole(button)
        if role == QDialogButtonBox.AcceptRole:
            self.do_save_config()

    def do_save_config(self):
        self.framework.set_raft_config_value('black_hole_network', bool(self.bhNetworkBox.isChecked()))
        if self.checkBoxUseProxy.isChecked():
            self.framework.set_raft_config_value('proxy_host', str(self.confProxyEdit.text()))
            self.framework.set_raft_config_value('proxy_port', int(self.confProxyPort.text()))
            self.framework.set_raft_config_value('use_proxy', True)
        else:
            self.framework.set_raft_config_value('use_proxy', False)

    def handle_useProxy_stateChanged(self, state):
        self.set_enable_proxy_edits()

    def set_enable_proxy_edits(self):
        if self.checkBoxUseProxy.isChecked():
            self.confProxyEdit.setEnabled(True)            
            self.confProxyPort.setEnabled(True)
        else:
            self.confProxyEdit.setEnabled(False)            
            self.confProxyPort.setEnabled(False)


